#ifdef __USAGE
usage: star_field [delay]
  delay -- optional, number of milliseconds to delay between refreshes
This is a star_field view screen saver.  
For use with QNX-Windows only.  
#endif

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <sys/sched.h>
#include <sys/psinfo.h>
#include <i86.h>
#include <windows/Qwindows.h>
#include <windows/Qtunnel.h>

#ifdef QW_H_TPP
#undef QW_H_TPP
#endif
#define QW_H_TPP 11

#ifdef QW_V_TPP
#undef QW_V_TPP
#endif
#define QW_V_TPP 11

typedef
struct point_d {
  long x, y;
  long dx, dy;
  long d2x, d2y;
  int age;
} Point_D;

#define TUNNEL
#define NUM_POINTS 60
#define SPEED 5
#define ACCEL 0.8
#define GROW 6

QW_BOX size;
int delay_val = 0;
Point_D point_data[NUM_POINTS];
#ifdef TUNNEL
 QWT_RECT rects[NUM_POINTS];
#else
 QW_CXY_COORD points[NUM_POINTS];
#endif
long h_max, v_max;
long h_mid, v_mid;

main( argc, argv )
int argc;
char *argv[];
{
	QW_EVENT_MSG event;

	if ( ! argc ) {
		printf("swarp:\n  A space warp screen saver.\n\n");
		printf("options: delay value swarp n to delay n/1000 seconds\n");
		exit(0);
	}

	if ( argc > 1 )
		delay_val = atoi( argv[1] );

	qnx_pflags(~0, _PPF_PRIORITY_FLOAT|_PPF_PRIORITY_REC, 0, 0);
	open_saver();
	setprio(0,3);
	
	srand( time(NULL) % RAND_MAX );
	
#ifdef TUNNEL
#ifdef __386__
    qwt_TunnelOpen( 0, 16384);
#else
    qwt_TunnelOpen( 4096, 16384);
#endif
#endif

	setup();
	
 	while (1) {
		if (EventWaiting()) {
			GetEvent(0,&event,sizeof(QW_EVENT_MSG));
			switch (Event(&event)) {
				case QW_TERMINATED:
				case QW_CLICK:
				case QW_QUIT:
				case QW_TYPED:
				case QW_CLOSED:
				case QW_CURSOR:
				case QW_DISMISS:
				case QW_MOVED:
					GraphicsClose(0);
					exit(0);
					break;
			}
		}
		move_points();
		if (delay_val) delay( delay_val );
	}
}

open_saver()
{
	if (!GraphicsOpen(NULL))
	{
	  fprintf(stderr, "QNX Windows use only\n");
	  exit(-1);
	}
	SetName("star_field","saver");
	
	ScreenInfo(NULL,NULL,NULL,NULL,&size,NULL,NULL,NULL);
	PictureOpen("scrnsve",NULL,NULL,QW_BLACK,0,NULL,NULL);
	WindowOpen("save",size.height,size.width,"f-OD-M-r-w;-OSc","Itx",NULL,0);
	CursorHide( );
}

setup()
{
   int i, radius, rad;
   double val;
   float sin_v, cos_v;
   
   radius = ((size.height/QW_V_TPP < size.width / QW_H_TPP )? 
             size.height/QW_V_TPP: size.width / QW_H_TPP )/2;
   h_max = ((long)size.width/QW_H_TPP)*100;
   v_max = ((long)size.height/QW_V_TPP)*100;
   h_mid = h_max/2;
   v_mid = v_max/2;

#ifdef TUNNEL
   qwt_SetSolidColor(1);
#endif
   for ( i=0; i < NUM_POINTS; i++ )
   {
     val = rand() * 6.2831853 / (double)RAND_MAX;  /* number between 0 and 2 Pi */
     sin_v = sin(val);
     cos_v = cos(val);
     point_data[i].dx  = (100.0 * SPEED * sin_v);
     point_data[i].d2x = (100.0 * ACCEL * sin_v);
     point_data[i].dy  = (100.0 * SPEED * cos_v);
     point_data[i].d2y = (100.0 * ACCEL * cos_v);
     
     rad = rand() % radius;
     point_data[i].x = point_data[i].dx / 2 * rad + h_mid;
     point_data[i].y = point_data[i].dy / 2 * rad + v_mid;
     point_data[i].age = 0;
#ifdef TUNNEL
     rects[i].Xul = point_data[i].x/100;
     rects[i].Yul = point_data[i].y/100;
     rects[i].Xlr = rects[i].Xul;
     rects[i].Ylr = rects[i].Yul;
     qwt_DrawFillRectangle( &(rects[i]) );
#else
     points[i].x = point_data[i].x/100;
     points[i].y = point_data[i].y/100;
     PixelRect( points[i].x, points[i].y, 1,1,0, QW_WHITE, QW_WHITE );
#endif
   }
#ifdef TUNNEL
   qwt_Flush( QWT_ASYNCH );
#else
   Draw();
#endif
}

move_points()
{
   int i, start;
   double val;
   float sin_v, cos_v;
   
#ifdef TUNNEL
   qwt_SetSolidColor(0);
   for ( i = 0; i < NUM_POINTS; i++ )
   {
     qwt_DrawFillRectangle( &(rects[i]) );
   }
   qwt_Flush( QWT_ASYNCH );
#endif

   for ( i = 0; i < NUM_POINTS; i++ )
   {
#ifndef TUNNEL
     PixelRect( points[i].x, points[i].y, 1+point_data[i].age/GROW,
                1+point_data[i].age/GROW,0, QW_BLACK, QW_BLACK );
#endif
     point_data[i].x += point_data[i].dx;
     point_data[i].y += point_data[i].dy;
     point_data[i].dx += point_data[i].d2x;
     point_data[i].dy += point_data[i].d2y;
     point_data[i].age++;
     if( (point_data[i].x <= 0) || (point_data[i].y <= 0) || 
         (point_data[i].x >= h_max ) || (point_data[i].y >= v_max ) )
     {
       val = rand() * 6.2831853 / (double)RAND_MAX;  /* number between 0 and 2 Pi */
       start = (rand() % 150) + 4;
       sin_v = sin(val);
       cos_v = cos(val);
       point_data[i].dx  = (100.0 * SPEED * sin_v);
       point_data[i].d2x = (100.0 * ACCEL * sin_v);
       point_data[i].dy  = (100.0 * SPEED * cos_v);
       point_data[i].d2y = (100.0 * ACCEL * cos_v);
       point_data[i].x = start*point_data[i].dx + h_mid;
       point_data[i].y = start*point_data[i].dy + v_mid;
       point_data[i].age = 0;
     }
#ifdef TUNNEL
     rects[i].Xul = point_data[i].x/100;
     rects[i].Yul = point_data[i].y/100;
     rects[i].Xlr = rects[i].Xul+point_data[i].age/GROW;
     rects[i].Ylr = rects[i].Yul+point_data[i].age/GROW;
     qwt_SetSolidColor(1);
     qwt_DrawFillRectangle( &(rects[i]) );
#else
     points[i].x = point_data[i].x/100;
     points[i].y = point_data[i].y/100;
     PixelRect( points[i].x, points[i].y, 1+point_data[i].age/GROW,
                1+point_data[i].age/GROW,0, QW_WHITE, QW_WHITE );
#endif
   }
#ifdef TUNNEL
   qwt_Flush( QWT_ASYNCH );
#else
   Draw();
#endif
}

